/**
  *  \file client/chart/prefixmode.cc
  *  \brief Starchart Mode: WPrefixChartMode
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/chart/prefixmode.h"
#include "gfx/palette.h"
#include "client/chartloc.h"
#include "ui/uiglobal.h"

WPrefixChartMode::WPrefixChartMode(int value)
    : WSimpleChartMode(ChartMode_Prefix),
      inp(value),
      blink(false),
      blink_count(0)
{ }

WPrefixChartMode::~WPrefixChartMode()
{ }

void
WPrefixChartMode::drawOverlays(GfxCanvas& can, GChartLocation& loc)
{
    // Other overlays below our text
    WSimpleChartMode::drawOverlays(can, loc);

    // Figure out location
    GfxPoint pt = loc.getViewport().getCenter();
    string_t text = inp.getText();
    int wi = fonts[FONT_FIXED]->getTextWidth(text) + 14;
    int he = fonts[FONT_FIXED]->getTextHeight(text) + 2;

    GfxRect r(pt.x - wi/2, pt.y - he/2 + 20, wi, he);

    // Draw.
    // FIXME: this is a copy of UIPrefixPopup::draw. Can we share this?
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);

    drawSolidBar(ctx, r, COLOR_FIRE + 29);

    ctx.setColor(COLOR_DARKYELLOW);
    drawHLine(ctx, r.x, r.getBottomY()-1, r.getRightX()-1);
    drawVLine(ctx, r.getRightX()-1, r.y, r.getBottomY()-2);

    ctx.setColor(COLOR_FIRE + 30);
    drawHLine(ctx, r.x+1, r.y, r.getRightX()-1);
    drawVLine(ctx, r.x, r.y, r.getBottomY()-2);

    ctx.setColor(COLOR_BLACK);
    ctx.useFont(*fonts[FONT_FIXED]);
    outText(ctx, r.x + 3, r.y + 1, text);

    if (!blink) {
        drawSolidBar(ctx, GfxRect(r.getRightX() - 10, r.getBottomY() - 5, 7, 2), COLOR_BLACK);
    }
}

bool
WPrefixChartMode::handleEvent(const UIEvent& event, GChartLocation& loc)
{
    switch (event.type) {
     case ev_Key:
        switch (inp.handleKey(event.key.code)) {
         case UIPrefixArgInput::Accepted:
            loc.markChanged();
            return true;
         case UIPrefixArgInput::Canceled:
            delete this;
            return true;
         default:
            if (getClassOfKey(event.key.code & ~ss_AllMod) == KeyNormal) {
                WChartMode* m = getNextMode();
                UIEvent newEvent = event;
                newEvent.key.arg = inp.getValue();
                loc.markChanged();
                delete this;
                return m && m->handleEvent(newEvent, loc);
            } else {
                // It's a shift code. Maybe someone wants to track it. Why not.
                return WSimpleChartMode::handleEvent(event, loc);
            }
        }

     case ev_Tick:
        blink_count += event.tick.elapsed;
        if (blink_count > 40) {
            blink = !blink;
            blink_count = 0;
            loc.markChanged();
        }
        return WSimpleChartMode::handleEvent(event, loc);

     default:
        // FIXME: what to do with the mouse?
        return false;
    }
}

void
WPrefixChartMode::startPrefixArg(WChartModeList& chart_modes, int value)
{
    chart_modes.pushMode(*new WPrefixChartMode(value));
}
