/**
  *  \file client/wscanchart.h
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WSCANCHART_H
#define PCC_V2_CLIENT_WSCANCHART_H

#include "ui/ui.h"
#include "gfx/gfx.h"
#include "game/coord.h"
#include "cpluslib/sigslot.h"
#include "game/lock.h"
#include "ui/widgets/tooltip.h"
#include "client/widgets/chartwidget.h"

class WScannerChartWidget : public WChartWidget {
    GPoint           scan_origin;
    GPoint           scan_target;
    SignalConnection conn_change;
    SignalConnection conn_options;
    UITooltipControl tt_ctrl;

    void drawScannerXor(GfxCanvas& can);
    void moveKey(int dx, int dy);
    void doTooltip(GfxPoint pt);
 protected:
    void drawPre(GfxCanvas& can);
    void drawPost(GfxCanvas& can);

    virtual void drawScanner(GfxCanvas& can);

    virtual void  lockQueryLocation(bool& hyperjumping, int& current_gravity_pid);
    virtual int32 lockQueryDistance(GPoint pt);
    virtual void  doItemLock(GPoint clicked, TLockMode mode, bool marked_only, bool optimize_warp);

    bool handleKey(UIEvent::Key k);
    bool handleOption(UIEvent::Key k);
 public:
    WScannerChartWidget();
    ~WScannerChartWidget();

    void setScannerOrigin(GPoint pt);
    void setScannerTarget(GPoint pt);

    bool handleEvent(const UIEvent& e, bool second_pass);
    void onStateChange(int state, bool enable);

    void doKeyboardMode();

    GPoint getScannerOrigin() const;
    GPoint getScannerTarget() const;

    GPoint clipScannerPosition(GPoint pt) const;

    virtual void userMoveScannerTo(GPoint pt);

    void getLayoutInfo(LayoutInfo& info);

    enum {
        sc_StyleChange  = st_Last*2,  ///< user can changes style (C-M-letter)
        sc_Zoomable     = st_Last*4,  ///< user can zoom (+/-)
        sc_Keyboard     = st_Last*8,  ///< user can use keyboard
        sc_ShipsOnly    = st_Last*16, ///< only lock on ships
        sc_KeyboardMode = st_Last*32  ///< keyboard mode (Y) active
    };

    Signal1<int> sig_moved;
    Signal1<int> sig_dblclick;
};


inline GPoint
WScannerChartWidget::getScannerOrigin() const
{
    return scan_origin;
}

inline GPoint
WScannerChartWidget::getScannerTarget() const
{
    return scan_target;
}

#endif
