/**
  *  \file game/checksum.h
  *  \brief Checksum Routines
  *
  *  Various checksum routines needed to validate game data.
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_CHECKSUM_H
#define PCC_V2_GAME_CHECKSUM_H

#include <cstddef>
#include "cpluslib/types.h"

class Directory;

enum GControlSection {
    ShipSection   = 1,
    PlanetSection = 2,
    BaseSection   = 3
};

/* checksum.cc */
int32 getChecksum(const char* data, std::size_t length);
int32 getChecksum(uint16 value);

/* check-ctl.cc */
int32 getControlEntry(GControlSection otyp, int oid);
void  setControlEntry(GControlSection otyp, int oid, int32 checksum);

int   loadControl(Directory& dir, int player);
void  saveControl(Directory& dir, int type);

void  initControl();
void  doneControl();

/* check-fizz.cc */
int32 getFizzEntry(GControlSection otyp, int player);
void  setFizzEntry(GControlSection otyp, int player, int32 checksum);

void  loadFizz(Directory& dir);
void  saveFizz(Directory& dir);

/* check-crc.cc */
uint32 getDigest(const char* data, std::size_t length, uint32 value = 0);

#endif
