%
%  Ship Namer
%
%  Use 'Namer.Load' to load a text file containing possible names.
%  (Call multiple times to load multiple files.)
%
%  Use 'Namer' to assign one such name to a ship.
%  This will only assign names that are not yet taken.
%
%  25/Apr/2014 by Stefan Reuther
%  10/Apr/2015 added auto-initialisation for use as plugin
%
%  Requires PCC2 1.99.22 or later (for arrays / ReDim),
%  1.99.25 for use as plugin.
%

Print "[namer.q 10/Apr/2015]"

% Array of possible names.
Dim Shared Namer.Names(0)

% Utility: append value to an array.
Sub Array.Push(a, val)
  ReDim a(Dim(a)+1)
  a(Dim(a)-1) := val
EndSub

% Load a file containing ship names.
Sub Namer.Load(file)
  Local fd := FreeFile()
  Local line
  Open file For Input as #fd
  Do
    Input #fd, line, True
    If IsEmpty(line) Then Break
    line := Trim(line)
    If line<>'' And Mid(line, 1, 1) <> ';' Then Array.Push Namer.Names, line
  Loop
  Close #fd
EndSub

% Initialize
Sub Namer.Init
  ReDim Namer.Names(0)
  Try Namer.Load MakeFileName(System.GameDirectory, "names.txt")
  If Dim(Namer.Names) = 0 Then
    Local plugdir = System.Plugin("namer").Directory
    Select Case My.Race$
      Case 1
        Namer.Load MakeFileName(plugdir, "fed.txt")
      Case 2
        Namer.Load MakeFileName(plugdir, "gorn.txt")
      Case 5
        Namer.Load MakeFileName(plugdir, "pirate.txt")
      Case 8
        Namer.Load MakeFileName(plugdir, "empire.txt")
      Case 9
        Namer.Load MakeFileName(plugdir, "robots.txt")
      Case 11
        Namer.Load MakeFileName(plugdir, "colony.txt")
      Case Else
        Print "Note: no ship names have been defined for automatic naming!"
    EndSelect
  EndIf
EndSub

% Assign name to a ship.
% Invoke from the ship.
Sub Namer
  Local i, n, sid := Id, did = False
  If Dim(Namer.Names) = 0 Then
    MessageBox "No ship names known. Please prepare a \"names.txt\" file with possible ship names to choose from."
  Else
    For i:=1 To 100 Do              % paranoia counter
      n := Random(0, Dim(Namer.Names))
      If IsEmpty(FindShip(Name = Namer.Names(n))) Then
        Name := Namer.Names(n)
        did := True
        Break
      EndIf
    Next
    If Not did Then
      MessageBox "I was unable to find a name for this ship"
    EndIf
  EndIf
EndSub

On Load Do Namer.Init
Bind Ship 'shift-n' := 'Namer'
