/**
  *  \file u/t_util_strtab.cc
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "util/strtab.h"
#include "u/t_util.h"

static const char*const table[]  = { "one", "two", "three", 0 };
static const char*const table2[] = { "eins", "zwei", "drei" };

void
UtilStrtabTestSuite::testStaticTable()
{
    {
        // Initialisation with null-terminated
        StaticStringTable t(table);
        TS_ASSERT_EQUALS(t.getFirstIndex(), 0);
        TS_ASSERT_EQUALS(t.getLastIndex(), 3);
        TS_ASSERT_EQUALS(t.getNextIndex(0), 1);
        TS_ASSERT_EQUALS(t.getNextIndex(1), 2);
        TS_ASSERT_EQUALS(t.getNextIndex(2), 3);
        TS_ASSERT_EQUALS(t(0), "one");
        TS_ASSERT_EQUALS(t(1), "two");
        TS_ASSERT_EQUALS(t(2), "three");

        // Out-of-range access must not crash, but exact value isn't defined. FIXME: should it?
        t(-1);
        t(3);
        t(4);
        t(5);
    }

    {
        // Initialisation with explicit size
        StaticStringTable t(table2, 3);
        TS_ASSERT_EQUALS(t.getFirstIndex(), 0);
        TS_ASSERT_EQUALS(t.getLastIndex(), 3);
        TS_ASSERT_EQUALS(t.getNextIndex(0), 1);
        TS_ASSERT_EQUALS(t.getNextIndex(1), 2);
        TS_ASSERT_EQUALS(t.getNextIndex(2), 3);
        TS_ASSERT_EQUALS(t(0), "eins");
        TS_ASSERT_EQUALS(t(1), "zwei");
        TS_ASSERT_EQUALS(t(2), "drei");

        t(-1);
        t(3);
        t(4);
        t(5);
    }

    {
        // Initialisation with subrange
        StaticStringTable t(table2, 2);
        TS_ASSERT_EQUALS(t.getFirstIndex(), 0);
        TS_ASSERT_EQUALS(t.getLastIndex(), 2);
        TS_ASSERT_EQUALS(t.getNextIndex(0), 1);
        TS_ASSERT_EQUALS(t.getNextIndex(1), 2);
        TS_ASSERT_EQUALS(t(0), "eins");
        TS_ASSERT_EQUALS(t(1), "zwei");

        TS_ASSERT_DIFFERS(t(2), "drei");

        t(-1);
        t(3);
        t(4);
        t(5);
    }

    {
        // Initialisation with blank
        StaticStringTable t(0, 0);
        TS_ASSERT_EQUALS(t.getFirstIndex(), 0);
        TS_ASSERT_EQUALS(t.getLastIndex(), 0);

        t(-1);
        t(0);
        t(1);
        t(2);
    }
}
