/**
  *  \file util/pluginstall.h
  *  \brief Plugin Installer
  */
#ifndef PCC_V2_UTIL_PLUGINSTALL_H
#define PCC_V2_UTIL_PLUGINSTALL_H

#include <memory>
#include "io/dirs.h"
#include "cpluslib/string.h"
#include "cpluslib/smartptr.h"
#include "io/streams.h"

class PluginManager;
class Plugin;

/** Plugin Installer.
    Allows installing and removing plugins. */
class PluginInstaller {
 public:
    enum ScanResult {
        NoPlugin,
        OnePlugin,
        MultiplePlugins
    };

    PluginInstaller(PluginManager& mgr, Directory& rootDir, ProgressMonitor& pm);

    ~PluginInstaller();

    Plugin* prepareInstall(string_t fileName);

    void doInstall(bool dry = false);

    ScanResult checkInstallAmbiguity(string_t& out);

    bool doRemove(Plugin* pPlug, bool dry = false);

 private:
    // Integration
    PluginManager& manager;
    Directory& rootDir;
    ProgressMonitor& progress;

    // State
    Ptr<Directory> srcDir;
    Ptr<Stream> srcFile;
    std::auto_ptr<Plugin> apPlug;
};

#endif
